from boiler.definitions import ActivityType

# This translates from the old activity names (prior to March 2020) to the new names.
activity_type_mapping = {
    'Abandon_Package': ActivityType.PERSON_ABANDONS_PACKAGE,
    'Carry_in_Hands': ActivityType.CARRY_IN_HANDS,
    'Carry_on_Back': ActivityType.CARRY_ON_BACK,
    'Casing_Facility': ActivityType.CASING_FACILITY,
    'Close_Facility_Door': ActivityType.PERSON_CLOSES_FACILITY_DOOR,
    'Close_Trunk': ActivityType.PERSON_CLOSES_TRUNK,
    'Close_Vehicle_Door': ActivityType.PERSON_CLOSES_VEHICLE_DOOR,
    'Embrace_Interaction': ActivityType.PERSON_EMBRACES_PERSON,
    'Enter_Facility': ActivityType.PERSON_ENTERS_SCENE_THROUGH_STRUCTURE,
    'Enter_Through_Structure': ActivityType.PERSON_ENTERS_SCENE_THROUGH_STRUCTURE,
    'Enter_Vehicle': ActivityType.PERSON_ENTERS_VEHICLE,
    'Exit_Facility': ActivityType.PERSON_EXITS_SCENE_THROUGH_STRUCTURE,
    'Exit_Through_Structure': ActivityType.PERSON_EXITS_SCENE_THROUGH_STRUCTURE,
    'Exit_Vehicle': ActivityType.PERSON_EXITS_VEHICLE,
    'Hand_Interaction': ActivityType.HAND_INTERACTS_WITH_PERSON,
    'Heavy_Carry': ActivityType.PERSON_CARRIES_HEAVY_OBJECT,
    'Joining_Queue': ActivityType.JOINING_QUEUE,
    'Laptop_Interaction': ActivityType.PERSON_INTERACTS_WITH_LAPTOP,
    'Load_Vehicle': ActivityType.PERSON_LOADS_VEHICLE,
    'Object_Transfer': ActivityType.PERSON_TRANSFERS_OBJECT,
    'Open_Facility_Door': ActivityType.PERSON_OPENS_FACILITY_DOOR,
    'Open_Trunk': ActivityType.PERSON_OPENS_TRUNK,
    'Open_Vehicle_Door': ActivityType.PERSON_OPENS_VEHICLE_DOOR,
    'People_Talking': ActivityType.PERSON_TALKS_TO_PERSON,
    'Pick_Up_Object': ActivityType.PERSON_PICKS_UP_OBJECT,
    'Purchasing': ActivityType.PERSON_PURCHASES,
    'Put_Down_Object': ActivityType.PERSON_PUTS_DOWN_OBJECT,
    'Read_Document': ActivityType.PERSON_READS_DOCUMENT,
    'Riding': ActivityType.PERSON_RIDES_BICYCLE,
    'Set_Down_Object': ActivityType.PERSON_PUTS_DOWN_OBJECT,
    'Sit_Down': ActivityType.PERSON_SITS_DOWN,
    'Stand_Up': ActivityType.PERSON_STANDS_UP,
    'Talk_on_Phone': ActivityType.PERSON_STANDS_UP,
    'Text_On_Phone': ActivityType.PERSON_TEXTS_ON_PHONE,
    'Theft': ActivityType.PERSON_STEALS_OBJECT,
    'Unload_Vehicle': ActivityType.PERSON_UNLOADS_VEHICLE,
    'Vehicle_DropsOff_Person': ActivityType.VEHICLE_DROPS_OFF_PERSON,
    'Vehicle_Moving': ActivityType.VEHICLE_MOVING,
    'Vehicle_PicksUp_Person': ActivityType.VEHICLE_PICKS_UP_PERSON,
    'Vehicle_Reversing': ActivityType.VEHICLE_REVERSES,
    'Vehicle_Starting': ActivityType.VEHICLE_STARTS,
    'Vehicle_Stopping': ActivityType.VEHICLE_STOPS,
    'Vehicle_Turning_Left': ActivityType.VEHICLE_TURNS_LEFT,
    'Vehicle_Turning_Right': ActivityType.VEHICLE_TURNS_RIGHT,
    'Vehicle_UTurn': ActivityType.VEHICLE_MAKES_U_TURN,
}
